function [est, confint, p, f, SSH] = rscomparative(y, coord, d, A, prefix, alfa)

% RSCOMPARATIVE - analyse comparative experiments. Version 0.1.24.
%
% The syntax is
% [est, confint, p, f, SSH] = rscomparative(y, coord, [d], [A], [prefix], [alpha])
%
% Arguments
% y is the vector of observations
% coord is a matrix of coordinates: which treatment, block, etc. is the corresponding observation from?
% [d] and [A] are voluntary arguments. If they are provided then the linear hypothesis A*est=d is tested.
% [prefix] is a string to determine the file name if plots from residual analysis are to be saved.
% [alpha] determines the confidence level of the 1-alpha confidence intervals for the estimated parameters.
%
% rscomparative uses the model
% y = tau1*z1 + ... + beta1*v1 + ... + gamma1*w1+ ... + epsilon,
% with the additional condition that the sum of the betas, the sum of the gammas etc. all equal zero.
%
% Return values
% est is the least squares estimate
% confint are 1-alpha-confidence intervals for the estimated parameters
% p is the p value for the linear hypothesis test
% f is the test statistics
% SSH is the hypothesis sum of squares
% 
% If you want to set [alpha] or [prefix] without testing a linear hypothesis, set A=[] and d to anything.
% If you want to set [alpha] without saving the pictures, set prefix=[].
%
% References
% Neter, Wasserman, Kutner: Applied Linear Regression Models, R.D. Irwin Inc., 1983, Chapter 9.


% Copyright (C) 2016 Mikael Kurula <mkurula@abo.fi>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
% FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, see <http://www.gnu.org/licenses/>.
%
% Author: Mikael Kurula <mkurula@abo.fi>, 2016


% Check input
if nargin < 2 || nargin > 6
    error('Try help rscomparative for instructions.') ;
end
if size(y,2) > 1,
    error('y has more than one column. Try help rscomparative for instructions.');
end
if nargin < 6,
  alfa=0.05;
end

% Build data matrix and augment response vector
[N, m]=size(coord);
X=[];
for j=1:m,
  co=(coord(:,j));
  n(j)=max(co);
  Id=eye(n(j));
  Xj=Id(co,:);
  X=[X Xj];
end
Xj=[];
for j=2:m,
  for l=1:m,
  if j==l,
    Xj=[Xj ones(1,n(l))];
  else
    Xj=[Xj zeros(1,n(l))];
  end
  end
  X=[X ; Xj];
end  
y=[y ; zeros(m-1,1)];

% Do regression and calculate confidence intervals
est=X\y;
yhat=X*est;
r=y-yhat;
SSE=r'*r;
dfs=n-1;
dfE=N-1-sum(dfs);
MSE=SSE/dfE;
C=inv(X'*X);           
d_i=sqrt(MSE*diag(C));
t_alfa_2=tinv(1-alfa/2,dfE);
confint=[est-d_i*t_alfa_2 est+d_i*t_alfa_2];

% Test linear hypothesis if asked for
if nargin > 3 && size(A,1)>0,
  [p, f, SSH] = rslinhyp(y, X, d, A);
else
  p=1; f=0; SSH=0;
end

%Calculate externally studentised residual
a=size(X,2);
H=X*C*X';
hii=diag(H);
Sisq=(dfE*MSE-r.^2./(1-hii))/(dfE-1);
d_i=sqrt(Sisq.*(1-hii));
t_alfa_2=tinv(1-alfa/2,dfE-1);
rint=[r-d_i*t_alfa_2 r+d_i*t_alfa_2];

% Clean and analyse residual
rint((end-(m-2)):end,:)=[];
X((end-(m-2)):end,:)=[];
y((end-(m-2)):end)=[];
r((end-(m-2)):end)=[];
yhat((end-(m-2)):end)=[];

% Take care of differences between Octave and Matlab
if exist ('OCTAVE_VERSION', 'builtin')       % Are we running Octave?
   graphics_toolkit('gnuplot')
   PprSz=[6 4];
else
   PprSz=[12 8];
end 

% Check that the residual is normally distributed
figure('PaperPosition',[0 0 PprSz],'PaperSize',PprSz);
normplot(r)
if nargin > 4 && size(prefix,1)>0,
   print([prefix,'-rscomparative-normplot'],'-dpdf')
end

% Draw scatter plots
Time=(1:(N-(m-2)))';

figure('PaperPosition',[0 0 PprSz],'PaperSize',PprSz);
plot(Time,r,'+','linewidth',2)
title('Residual vs. time')
xlabel('Time 1:N (table order of X)')
ylabel('Residual')
xlim([0 N+1]);
if nargin > 4 && size(prefix,1)>0,
   print([prefix,'-rscomparative-time'],'-dpdf')
end

figure('PaperPosition',[0 0 PprSz],'PaperSize',PprSz);
plot(yhat,r,'+','linewidth',2)
title('Residual vs. predicted response')
xlabel('Predicted response y-hat')
ylabel('Residual')

xvar=yhat;
if min(xvar)>0
  Lo=0.95*min(xvar);
elseif min(xvar)<0
  Lo=1.05*min(xvar);
else
  Lo=-0.05;
end
if max(xvar)<0
  Up=0.95*max(xvar);
elseif max(xvar)>0
  Up=1.05*max(xvar);
else
  Up=0.05;
end
xlim([Lo Up]);

if nargin > 4 && size(prefix,1)>0,
   print([prefix,'-rscomparative-pred'],'-dpdf')
end

for i=1:m,
  vari=coord(:,i);
  cor(1+i,1)=corr(r,vari);
  figure('PaperPosition',[0 0 PprSz],'PaperSize',PprSz);
  plot(vari,r,'+','linewidth',2)
  title(['Residual vs. coordinate #',int2str(i)])
  xlabel(['Coordinate #',int2str(i)])
  ylabel('Residual')

  xvar=vari;
  if min(xvar)>0
    Lo=0.95*min(xvar);
  elseif min(xvar)<0
    Lo=1.05*min(xvar);
  else
    Lo=-0.05;
  end
  if max(xvar)<0
    Up=0.95*max(xvar);
  elseif max(xvar)>0
    Up=1.05*max(xvar);
  else
    Up=0.05;
  end
  xlim([Lo Up]);

  if nargin > 4 && size(prefix,1)>0,
    print([prefix,'-rscomparative-coord',int2str(i)],'-dpdf')
  end
end

%Finally, plot residual with confidence intervals
figure('PaperPosition',[0 0 PprSz],'PaperSize',PprSz);
h=errorbar(r, (rint(:,2)-rint(:,1))/2,'+');
set(h,'linewidth',2)
hold on
plot([0 N+1],[0 0],'r','linewidth',2)
hold off
title('Residual with 0.95-confidence intervals')
xlabel('Time 1:N (table order of X)')
ylabel('Residual')
xlim([0 N+1]);
if nargin > 4 && size(prefix,1)>0,
   print([prefix,'-rsresid-confint'],'-dpdf')
end

end