function [pred, confint_mean, confint_newobs] = rspredict(y, X, x, alpha)
% RSPREDICT - Use linear regression to predict the response in a given point x, including a 1-alpha confidence interval for the mean response. Version 0.1.18.
%
% The syntax is
% [pred, confint_mean, confint_newobs] = rspredict(y, X, x, [alpha])
%
% Arguments
% y is an N by 1 column vector of data to fit.
% X is an N by a matrix containing the values of a predictors used in the regression, including a possible constant 1 column.
% x is a 1 by a row vector with the factor combination where one wants to predict.
% alpha is an optional parameter specifying the significance level to use for the confidence interval. If omitted, then alpha=0.05 is assumed.
%
% Return values
% pred is the predicted response.
% confint_mean is a 1 by 2 row vector containing the end points of the symmetric (1-alpha)-confidence interval for the expected mean response in the given point. The width is zero if the model fits the data prefectly.
% confint_newobs is the corresponding confidence interval for a single new prediction at the relevant point. This is wider than the previous interval (for the mean of infinitely many new predictions).
%
% References
% Neter-Wasserman-Kutner: Applied Linear Regression Models, Richard D. Irwin Inc., 1983, sections 3.4-3.5, 7.6-7.7.
% Draper-Smith: Applied Regression Analysis, 2nd ed, Wiley 1981, pp. 210-211.
% Myers, Montgomery & Anderson-Cook: Response Surface Methodology, 3rd ed., Wiley, 2009, Section 2.5.3.


% Copyright (C) 2016 Mikael Kurula <mkurula@abo.fi>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
% FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, see <http://www.gnu.org/licenses/>.
%
% Author: Mikael Kurula <mkurula@abo.fi>, 2015


% Check input
if nargin < 3 || nargin > 4,
    error('Try help rspredict for instructions.') ;
end
if size(y,2) > 1,
    error('y has more than one column. Try help rspredict for instructions.');
end
if size(x,1) > 1,
    error('x has more than one row. Try help rspredict for instructions.');
end
if cond(X'*X) > 1e10,
    warning('The normal equation is ill-conditioned. If the matrix X does not have linearly dependent columns, try standardizing the factor data.');
end

% Linear regression and prediction
b=X\y;
r=y-X*b;
[N,a]=size(X);
nu=N-a;
MSE=r'*r/nu;
pred=x*b;
se_mean=sqrt(MSE*x*(X'*X)^-1*x');
se_new=sqrt(MSE+MSE*x*(X'*X)^-1*x');
if nargin == 3,
  alpha=0.05
end
quant=tinv(1-alpha/2,nu);
confint_mean=pred+[-se_mean*quant, se_mean*quant];
confint_newobs=pred+[-se_new*quant, se_new*quant];
end
